// ==UserScript==
// @name         Wenku Doc Downloader
// @namespace    http://tampermonkey.net/
// @version      1.3.6
// @description  下载“百度文库”文档，导出txt或pdf。支持①豆丁网②爱问共享资料（新浪文档）③得力文库④道客巴巴，文档导出pdf。在文档页面最最底部有蓝/绿色长方形按钮，说明脚本生效了。2021/12/17百度文档更新了技术，【脚本对大部分百度文档无效了】，请等待以后更新，抱歉。
// @author       allenlv2690@gmail.com
// @match        https://wenku.baidu.com/view/*
// @match        https://www.docin.com/p-*
// @match        https://ishare.iask.sina.com.cn/f/*
// @match        https://www.deliwenku.com/p-*
// @match        *://www.doc88.com/p-*
// @match        https://wk.baidu.com/view/*
// @require      https://cdn.bootcdn.net/ajax/libs/FileSaver.js/2.0.5/FileSaver.js
// @require      https://cdn.bootcdn.net/ajax/libs/jszip/3.6.0/jszip.js
// @require      https://cdn.bootcdn.net/ajax/libs/jspdf/2.3.1/jspdf.umd.min.js
// @icon         https://wenku.baidu.com/favicon.ico
// @grant        none
// @license      GPL-3.0-only
// @create       2021-11-22
// ==/UserScript==

"use strict";

let options = {
    "fast_mode": false
}

const jsPDF = jspdf.jsPDF;

/*
 *  附属功能函数部分
 */

/**
 * 创建并下载文件
 * @param {string} fileName 
 * @param {string} content 
 */
function createAndDownloadFile(fileName, content) {
    let aTag = document.createElement('a');
    let blob = new Blob([content]);
    aTag.download = fileName;
    aTag.href = URL.createObjectURL(blob);
    aTag.click();
    URL.revokeObjectURL(blob);
}

/**
 * 允许打印页面
 */
function allowPrint() {
    let style = document.createElement("style");
    style.innerHTML = `@media print {
        body{
            display:block;
        }
    }`;
    document.getElementsByTagName("head")[0].appendChild(style);
}

/**
 * 隐藏按钮，打印页面，显示按钮
 */
function hideBtnThenPrint() {
    // 隐藏按钮，然后打印页面
    let section = document.getElementsByClassName("btns_section")[0];
    section.style.display = "none";
    window.print();
    // 打印结束，显示按钮
    section.style.removeProperty("display");
}

/**
 * 增强按钮（默认为蓝色按钮：展开文档）的点击效果
 * @param {String} custom_btn 按钮变量名
 */
function enhanceBtnClickReaction(custom_btn = null) {
    let aim_btn;
    // 如果不使用自定义按钮元素，则默认为使用蓝色展开文档按钮
    if (!custom_btn || custom_btn === "btn_1") {
        aim_btn = document.getElementsByClassName("init-btn")[0];
    } else if (custom_btn === "btn_2") {
        aim_btn = document.querySelector(".save-doc-btn");
    } else if (custom_btn === "btn_3") {
        aim_btn = document.querySelector(".save-html-btn");
        console.log(aim_btn);
    }

    let old_color = aim_btn.style.color; // 保存旧的颜色
    let old_text = aim_btn.textContent; // 保存旧的文字内容
    // 暗红缩小
    aim_btn.style.color = "#c90000";
    aim_btn.style.fontWeight = "normal";
    aim_btn.textContent = `——>[ ${old_text} ]<——`;
    // 复原加粗
    let changeColor = function() {
        aim_btn.style.color = old_color;
        aim_btn.style.fontWeight = "bold";
        aim_btn.textContent = old_text;
    };
    setTimeout(changeColor, 1500);
}

/**
 * 用input框跳转到对应页码
 * @param {Element} cur_page 当前页码
 * @param {string} aim_page 目标页码
 * @param {string} event_type 键盘事件类型："keyup" | "keypress" | "keydown"
 */
function jump2pageNo(cur_page, aim_page, event_type) {
    // 设置跳转页码为目标页码
    cur_page.value = aim_page;
    // 模拟回车事件来跳转
    let keyboard_event_enter = new KeyboardEvent(event_type, {
        bubbles: true,
        cancelable: true,
        keyCode: 13
    });
    cur_page.dispatchEvent(keyboard_event_enter);
}

/**
 * 滚动到页面底部
 */
function scrollToBottom() {
    window.scrollTo({
        top: document.body.scrollHeight,
        behavior: "smooth"
    });
}

/**
 * 用于纯文本文档的文本美化，适用性：百度文库
 * @param {string} text 
 * @returns 美化后的文本
 */
function formatText(text) {
    let reg_exp_1 = new RegExp(" ?[(]?=[\u4e00-\u9fa5] ?[)]");
    let reg_exp_2 = new RegExp("(?<=TEMP[\u4e00-\u9fa5]) ?");

    let text_1 = text.replace(reg_exp_1, "TEMP");
    let text_2 = text_1.replace(reg_exp_2, "");
    let text_3 = text_2.replace("TEMP", "");
    let text_final = text_3.replace(/ /g, " ");
    return text_final;
}

/**
 * 用于图形文字混合型文档的文本美化，适用性：百度文库
 * @param {string} text
 * @returns 美化后的文本
 */
function formatText2(text) {
    let reg_exp = new RegExp("[  ]{2,}");
    let content_1 = text.replace(reg_exp, "\n");

    let content_2 = content_1.replace(/[  ]\n/g, "\n");

    let reg_exp_2 = new RegExp("\n[   ]*\n*\n");
    let content_3 = content_2.replace(reg_exp_2, "\n");

    let reg_exp_3 = new RegExp(" *\n * ");
    let content_4 = content_3.replace(reg_exp_3, "\n");

    let content_5 = content_4.replace(/[  ]/g, " ");
    let final_content = content_5.replace(/[ \n]精选文档[ \n]/g).replace(/\n{2,}/g, "\n");

    return final_content;
}

/**
 * 判断文档的详细类型，适用性：百度文库
 * @returns 源文档类型-实际内容类型，如: doc-pic-word
 */
function detectType() {
    // 获取文档类型名称
    let file_type, type;
    try {
        let doc_title_wrap = document.getElementsByClassName("doc-title-wrap")[0];
        file_type = doc_title_wrap.children[0].className;
    } catch (e) {
        alert("请刷新页面以激活该按钮。\n先点击【导出pdf】橙色按钮后该按钮将无法使用。");
        return "safe_quit";
    }
    // 判断文档类型
    if (file_type.search("word") !== -1) {
        type = "word";
    } else if (file_type.search("ppt") !== -1) {
        type = "ppt";
    } else if (file_type.search("excel") !== -1) {
        type = "excel";
    } else if (file_type.search("pdf") !== -1) {
        type = "pdf";
    } else if (file_type.search("txt" !== -1)) {
        type = "txt";
    } else {
        type = file_type;
    }
    // 分别尝试获取相应元素列表，若列表长度为0则不存在相应元素，否则存在
    let pic_nums = document.getElementsByClassName("reader-pic-item").length;
    let word_nums = document.getElementsByClassName("reader-word-layer").length;
    let ppt_img_nums = document.getElementsByClassName("ppt-image-wrap").length;

    // 判断文档类型、文字和图片的数量状况
    if ((type === "ppt" && ppt_img_nums > 1) || (type === "pdf" && !word_nums && !pic_nums && ppt_img_nums)) {
        // ppt: 包含至少2页内容 / 纯ppt图形页面构成
        return "ppt";
    } else {
        return type;
    }
}

/**
 * 用try移除元素
 * @param {Element} element 要移除的元素
 */
function tryToRemoveElement(element) {
    try {
        element.remove();
    } catch (e) {
        console.log();
    }
}

/**
 * 用try移除 [元素列表1, 元素列表2, ...] 中的元素
 * @param {Array} elem_list_box 要移除的元素列表构成的列表
 */
function tryToRemoveSameElem(elem_list_box) {
    for (let elem_list of elem_list_box) {
        if (!elem_list) {
            continue;
        }
        for (let elem of elem_list) {
            try {
                elem.remove();
            } catch (e) {
                console.log();
            }
        }
    }
}

/**
 * 使文档在页面上居中
 * @param {String} class_name 文档元素的class
 * @param {String} default_offset 文档部分向右偏移的百分比（0-59）
 * @returns 偏移值是否合法
 */
function centerDoc(class_name, default_offset) {
    let doc_main = document.getElementsByClassName(class_name)[0];
    let offset = window.prompt("请输入偏移百分位:", default_offset);
    // 如果输入的数字不在 0-59 内，提醒用户重新设置
    if (offset.length === 1 && offset.search(/[0-9]/) !== -1) {
        doc_main.style.marginLeft = offset + "%";
        return true;
    } else if (offset.length === 2 && offset.search(/[1-5][0-9]/) !== -1) {
        doc_main.style.marginLeft = offset + "%";
        return true
    } else {
        alert("请输入一个正整数，范围在0至59之间，用来使文档居中\n（不同文档偏移量不同，所以需要手动调整）");
        return false;
    }
}

/**
 * 提示文档已经展开，改变按钮形态：隐去蓝色按钮，显示绿色按钮
 * @param {Boolean} shrink_btn2 是否缩小绿色按钮
 * @param {Boolean} recommend_btn2 是否推荐绿色按钮
 * @param {String} btn2_new_text （如果需要）替换绿色按钮的文本为新文本
 * @param {Boolean} use_hint 是否提示文档已经展开
 */
function transformButtons(shrink_btn2 = false, recommend_btn2 = false, btn2_new_text = "", use_hint = true) {
    // 提示文档已经展开
    if (use_hint) {
        let hint = "文档已经完全展开，可以导出";
        alert(hint);
    }
    // 准备调整按钮，先获取按钮
    let btn_1 = document.getElementsByClassName("init-btn")[0];
    let btn_2 = document.getElementsByClassName("save-doc-btn")[0];
    // 调整按钮显示状况
    btn_1.style.display = "none";
    btn_2.style.removeProperty("display");
    if (shrink_btn2) {
        btn_2.style.width = "34.8%";
    }
    if (btn2_new_text) {
        btn_2.textContent = btn2_new_text;
    }
    if (recommend_btn2) {
        btn_2.textContent += "（推荐）";
    }
}

/*
 *  主要功能函数部分
 */

/**
 * 清理并打印得力文库的文档页
 */
function printPageDeliwenku() {
    // 移除页面上的无关元素
    let selector = ".hr-wrap, #readshop, .nav_uis, .bookdesc, #boxright, .QQ_S1, .QQ_S, #outer_page_more, .works-manage-box.shenshu, .works-intro, .mt10.related-pic-box, .mt10.works-comment, .foot_nav, .siteInner";
    let elem_list = document.querySelectorAll(selector);
    for (let elem of elem_list) {
        tryToRemoveElement(elem);
    }
    // 修改页间距
    let outer_pages = document.getElementsByClassName("outer_page");
    for (let page of outer_pages) {
        page.style.marginBottom = "20px";
    }
    // 使文档居中
    alert("建议使用:\n偏移量: 3\n缩放: 112\n请上下滚动页面，确保每页内容都加载完成以避免空白页\n如果预览时有空白页或文末有绿色按钮，请取消打印重试");
    if (!centerDoc("boxleft", "3")) {
        return; // 如果输入非法，终止函数调用
    }
    // 打印文档
    hideBtnThenPrint();
}

/**
 * 清理并打印爱问共享资料的文档页
 * @returns 如果输入偏移量非法，返回空值以终止函数
 */
function printPageiShare() {
    // # 清理并打印爱问共享资料的文档页
    // ## 移除页面上无关的元素
    // ### 移除单个元素
    let topbanner = document.getElementsByClassName("detail-topbanner")[0];
    let header = document.getElementsByClassName("new-detail-header")[0];
    let fixright = document.getElementById("fix-right");
    let redpacket = document.getElementsByClassName("loginRedPacket-dialog")[0];
    let fixedrightfull = document.getElementsByClassName("fixed-right-full")[0];
    let footer = document.getElementsByClassName("website-footer")[0];
    let guess = document.getElementsByClassName("guess-you-like-warpper")[0];
    let detailtopbox = document.getElementsByClassName("detail-top-box")[0];
    let fullscreen = document.getElementsByClassName("reader-fullScreen")[0];
    let endhint = document.getElementsByClassName("endof-trial-reading")[0];
    let crumb_arrow;
    try { crumb_arrow = document.getElementsByClassName("crumb-arrow")[0].parentElement; } catch (e) { console.log(); }
    let copyright = document.getElementsByClassName("copyright-container")[0];
    let state_btn = document.getElementsByClassName("state-bottom")[0];
    let comments = document.getElementsByClassName("user-comments-wrapper")[0];
    // ### 执行移除
    let elem_list = [
        topbanner,
        header,
        fixright,
        redpacket,
        fixedrightfull,
        footer,
        guess,
        detailtopbox,
        fullscreen,
        endhint,
        crumb_arrow,
        copyright,
        state_btn,
        comments
    ];
    for (let elem of elem_list) {
        tryToRemoveElement(elem);
    }
    // ### 移除全部同类元素
    let elem_list_2 = document.querySelectorAll(".tui-detail, .adv-container")
    for (let elem_2 of elem_list_2) {
        tryToRemoveElement(elem_2);
    }
    // 使文档居中
    alert("建议使用:\n偏移量: 18\n缩放: 默认\n如果预览中有广告，就取消打印\n再点一次按钮，预览中应该就没有广告了");
    if (!centerDoc("doc-main", "18")) {
        return; // 如果输入非法，终止函数调用
    }
    // 隐藏按钮，然后打印页面
    hideBtnThenPrint();
}

/**
 * 清理并打百度文库的文档页
 * @returns 如果输入偏移量非法，返回空值以终止函数
 */
function printPageBaidu() {
    // # 清理并打百度文库的文档页
    // ## 移除无关页面元素
    // ### 要移除的单个元素
    let header_wrapper = document.getElementsByClassName("header-wrapper")[0];
    let right_wrapper = document.getElementById("right-wrapper-id");
    let reader_topbar = document.getElementsByClassName("reader-topbar")[0];
    let end_fold_page = document.getElementsByClassName("try-end-fold-page")[0];
    let catalog = document.querySelector(".catalog");
    let bottom = document.querySelector(".journal");

    let elem_list = [
        header_wrapper,
        right_wrapper,
        reader_topbar,
        end_fold_page,
        catalog,
        bottom
    ]
    for (let elem of elem_list) {
        tryToRemoveElement(elem);
    }
    // ### 移除全部同类元素
    let lazy_load_list = document.getElementsByClassName("lazy-load");
    let no_full_screen_list = document.getElementsByClassName("no-full-screen");
    let ads = document.getElementsByClassName("hx-warp");

    tryToRemoveSameElem([lazy_load_list, ads, no_full_screen_list]);

    // ## 文档类型测试
    // ### 常规文档测试
    let normal = Boolean(document.getElementsByClassName("left-wrapper")[0]);
    // ### 文献/期刊文档测试
    let journal = Boolean(document.getElementById("#journal-view"))
    let class_name;
    if (normal) {
        class_name = "left-wrapper";
    } else if (journal) {
        class_name = "right-wrapper";
    } else {
        alert("文档部分元素class不为left-wrapper，且id不为journal-view\n是未知文档类型，无法处理，请联系作者\nQQ: 2690874578\nEmail: allenlv2690@gmail.com");
    }

    // 使文档居中
    alert("建议使用:\n偏移量: 0\n缩放: 118%\n请上下滚动页面，确保每页内容都加载完成以避免空白页\n如果预览时有空白页或文末有按钮，请取消打印重试");
    if (!centerDoc(class_name, "0")) {
        return; // 如果输入非法，退出函数调用
    }
    // 隐藏按钮，然后打印页面
    hideBtnThenPrint();
}

function printPageWK() {
    // 去水印
    document.querySelectorAll("[class*=__wm]").forEach((elem) => {
        elem.className += " __web-inspector-hide-shortcut__";
    });
    // 调整页面边距、圆角、背景色
    let pages = document.querySelectorAll(".retype-page");
    pages.forEach((page) => {
        page.style.borderRadius = "0";
        page.style.background = "";
        page.style.padding = "0 0 0 0";
    });
    // 页面间隔调为0
    document.querySelectorAll(".gap").forEach((gap) => {
        gap.style.height = "0";
    });
    hideBtnThenPrint();
}

/**
 * 清理WK页面的无效元素
 */
function clearPageWK() {
    // 去除元素
    let selector = ".vip-cashier-dialog-wrap, .top-card, .college-strong-guide-contain, .reader-pop-manager-view-containter, .middle-box-root";
    document.querySelectorAll(selector).forEach((elem) => {
        tryToRemoveElement(elem);
    });
    // 去除父级元素
    let child_selector = ".bartop, .barbottom";
    document.querySelectorAll(child_selector).forEach((elem) => {
        tryToRemoveElement(elem.parentElement);
    });
    // 前项元素去除
    let previous_selector = ".reader-pop-manager-view-containter";
    document.querySelectorAll(previous_selector).forEach((elem) => {
        tryToRemoveElement(elem.previousElementSibling);
    });
}

/**
 * 创建“打印页面到PDF”按钮
 * @param {Function} printPageWebsite 
 * @returns btn_3元素引用
 */
function createPrintPageBtn(printPageWebsite) {
    let btn_3 = document.createElement("button");
    // 样式设定
    btn_3.setAttribute("class", "save-html-btn");
    btn_3.style.height = "25px";
    btn_3.style.width = "15%";
    btn_3.style.marginLeft = "0.2%";
    btn_3.style.backgroundColor = "orange";
    btn_3.style.border = "none";
    btn_3.textContent = "导出pdf";
    btn_3.style.color = "black";
    btn_3.style.fontWeight = "bold";
    btn_3.style.borderRadius = "10%";
    btn_3.style.zIndex = "99999";
    // 绑定事件，添加到页面上
    btn_3.onclick = () => {
        enhanceBtnClickReaction("btn_3");
        printPageWebsite();
    };
    let section = document.getElementsByClassName("btns_section")[0];
    section.appendChild(btn_3);
    return btn_3;
}

/**
 * 点击“继续阅读”，适用性：得力文库
 */
function readAllDeliwenku() {
    // 点击“同意并开始预览全文”
    let start_btn = document.getElementsByClassName("pre_button")[0];
    let display = start_btn.parentElement.parentElement.style.display;
    // 如果该按钮显示着，则点击，然后滚动至页面底部，最后终止函数
    if (!display) {
        start_btn.children[0].click();
        setTimeout("scroll(0, document.body.scrollHeight)", 200);
        return;
    }
    // 增强按钮点击效果
    enhanceBtnClickReaction();

    let read_all_btn = document.getElementsByClassName("fc2e")[0];
    let display2 = read_all_btn.parentElement.parentElement.style.display
        // 继续阅读
    if (display2 !== "none") {
        // 获取input元素
        let cur_page = document.querySelector("#pageNumInput");
        let page_old = cur_page.value;
        let page_max = cur_page.parentElement.nextElementSibling.textContent.replace(" / ", "");
        // 跳转到尾页
        jump2pageNo(cur_page, page_max, "keydown");
        // 跳转回来
        jump2pageNo(cur_page, page_old, "keydown");

        // 切换按钮准备导出
    } else {
        // 推荐导出全部图片链接
        transformButtons(true, true);
        // btn_3 橙色按钮
        createPrintPageBtn(printPageDeliwenku);
    }
}

/**
 * 点击“展开继续阅读”，适用性：爱尚共享资料
 */
function readAlliShare() {
    // 获取“继续阅读”元素
    let red_btn = document.getElementsByClassName("red-color")[0];
    let red_text = red_btn.textContent;
    // 增强按钮点击效果
    enhanceBtnClickReaction();
    // 如果可以展开，则展开
    if (red_text.search("点击可继续阅读") !== -1) {
        red_btn.click();
        setTimeout(readAlliShare, 1000);
    }
    // 否则启动按钮2，准备清理页面然后打印为PDF
    else {
        // 平滑往返页面顶部和底部
        window.scrollTo({
            top: 0,
            behavior: "smooth"
        });

        // 推荐导出全部图片链接
        transformButtons(true, true);
        // btn_3，橙色按钮
        createPrintPageBtn(printPageiShare);

        // 显示svg图片的链接
        let page1 = document.querySelector('[data-num="1"] .data-detail embed');
        if (!page1) {
            // 如果不存在svg图形，终止后续代码
            return;
        }
        let page2 = document.querySelector('[data-num="2"] .data-detail embed');
        let [svg1_src_div, svg2_src_div] = [document.createElement("div"), document.createElement("div")];
        svg1_src_div.innerHTML = `<div id="src-1"
                                    style="font-weight: bold;font-size: 20px; height: 100px; width: 100%">
                                        访问以下链接以复制文字:<br>${page1.src}
                                    </div>`;
        svg2_src_div.innerHTML = `<div id="src-1"
                                    style="font-weight: bold;font-size: 20px; height: 100px; width: 100%">
                                    访问以下链接以复制文字:<br>${page2.src}
                                    </div>`;
        // 添加到页面上
        page1.parentElement.parentElement.parentElement.append(svg1_src_div);
        page2.parentElement.parentElement.parentElement.append(svg2_src_div);
    }
}


/**
 * 点击“继续阅读”，适用性：百度文库
 * @returns 特殊文档类型不予下载，返回null
 */
function readAll() {
    // 获取“继续阅读”按钮
    let read_all_btn = document.getElementsByClassName("read-all")[0];
    // 如果存在“继续阅读”的按钮
    if (read_all_btn) {
        // 跳转到文末（等同于展开全文）
        let cur_page = document.getElementsByClassName("cur-page")[0];
        // 取得最大页码
        let page_max = cur_page.parentElement.children[2];
        // 设置跳转页码为最大页码
        cur_page.value = page_max;
        // 跳转到尾页
        jump2pageNo(cur_page, page_max, "keyup");

        // 否则认为已经展开了文档，判断文档类型，然后显示对应的按钮
    } else {
        let type;
        try {
            // 判断文档类型
            type = detectType();
            console.log(type);
        } catch (e) {
            alert("未知/特殊文档类型，例如学术文献，暂不支持下载\n也可与作者反馈或联系:\nallenlv2690@gmail.com");
            return null;
        }
        // 添加 btn_3
        createPrintPageBtn(printPageBaidu);
        // ppt、pdf有部分文档是纯图片的
        if (type === "ppt") {
            transformButtons(true, true, "导出全部图片链接");
        }
        // 其他均为canvas图形
        else {
            transformButtons(true);
            // 禁用按钮
            document.querySelector(".save-doc-btn").disabled = true;
            document.querySelector(".save-html-btn").disabled = true;
            // 监听scroll，捕获canvas
            let storeCanvases_Baidu = () => {
                let selector = ".creader-canvas";
                let id_format = "creader-canvas-@";
                storeCanvases(selector, id_format, max_id);
            }
            window.onscroll = storeCanvases_Baidu;
        }
    }
}

function readAllDoc88() {
    // 获取“继续阅读”按钮
    let continue_btn = document.querySelector("#continueButton");
    // 增强按钮点击效果
    enhanceBtnClickReaction();
    // 如果存在“继续阅读”按钮
    if (continue_btn) {
        // 跳转到文末（等同于展开全文）
        let cur_page = document.querySelector("#pageNumInput");
        // 取得最大页码
        let page_max = cur_page.parentElement.textContent.replace(" / ", "");
        // 跳转到尾页
        jump2pageNo(cur_page, page_max, "keypress");
        // 返回顶部
        setTimeout(jump2pageNo(cur_page, "1", "keypress"), 1000);
    }
    // 否则启动按钮2
    else {
        transformButtons(true, false, "", true);
        // 显示btn_3
        document.querySelector(".save-html-btn").style.removeProperty("display");
    }
}

function readAllWK() {
    enhanceBtnClickReaction();
    let first_open = document.querySelector(".open-arrow");
    // 第一次展开文档
    if (first_open) {
        first_open.click();
        setTimeout(readAllWK, 1000);
    }
    // 非第一次展开文档
    else {
        let read_all = document.querySelector(".pagerwg-button");
        // 如果“继续阅读”按钮不存在或已经被隐藏
        if (!read_all || read_all.style.display === "none") {
            // 尝试关闭弹窗（如果存在）
            try {
                functiondocument.querySelector(".btn-cancel[id*=wui]").click();
            } catch (e) { console.log("继续阅读按钮引起的弹窗不存在，无需关闭"); }
            // 尝试移除底部无关元素
            let bottom_part = document.querySelector(".live-broadcast-pop-wrap");
            tryToRemoveElement(bottom_part.previousElementSibling);
            // 换为按钮2
            transformButtons(false, false, "", true);
        }
        // 否则文档尚未完全展开，继续点击按钮展开文档
        else {
            setTimeout(() => {
                read_all.click();
                clearPageWK();
                readAllWK();
            }, 1000);
        }
    }
}

/**
 * 存储非PPT文档的png图形链接，适用性：百度文库
 */
function savePDFData() {
    let pic_urls = document.getElementsByClassName("reader-pic-item");
    let text_list = [];
    // 去掉前缀
    let reg_exp_1 = new RegExp(": ?url[(]");
    // 去掉后缀
    let reg_exp_2 = new RegExp("[)]; ?background-position");

    for (let i = 0; i < pic_urls.length; i++) {
        let whole_text = pic_urls[i].getAttribute("style");
        let de_pretext = whole_text.split(reg_exp_1)[1];
        let url = de_pretext.split(reg_exp_2)[0];
        text_list.push(url);
    }

    text_list[0] = text_list[0].replace(/"/g, "");
    let content = text_list.join("\n");
    // 启动下载
    createAndDownloadFile("urls.csv", content);
}

/**
 * 存储纯文本，适用性：百度文库
 */
function saveDocData() {
    // 获取文本
    let text_elements = document.getElementsByClassName("reader-word-layer");
    let texts = [];
    for (let elem of text_elements) {
        texts.push(elem.textContent);
    }
    // 美化后导出文本
    let origin_content = texts.join("");
    let content = formatText(origin_content);
    createAndDownloadFile("纯文本文档.txt", content);
}

/**
 * 存储PPT图像链接，适用性：百度文库
 */
function savePPTData() {
    let pic_elements = document.getElementsByClassName("ppt-image-wrap");
    let pic_urls = [];

    for (let elem of pic_elements) {
        let pic_obj = elem.children[0];
        let url = pic_obj.src;
        pic_urls.push(url);
    }
    let content = pic_urls.join("\n");
    // 启动下载
    createAndDownloadFile("urls.csv", content);
}

/**
 * 存储文字型表格，适用性：百度文库
 */
function saveExcelData() {
    // 1. 拿到表格
    let table_pic = document.getElementsByClassName("reader-pic-item")[0];
    let url = table_pic.style.getPropertyValue("background-image");
    // 获取图片地址
    let pure_url = url.slice(5, -2);

    // 2. 拿到表格内文字信息
    let text_elems = document.getElementsByClassName("reader-word-layer");
    let text_list = [];
    for (let elem of text_elems) {
        text_list.push(elem.textContent);
    }
    let _text = text_list.join("\n");
    // 替换奇怪的空格
    let text = _text.replace(/ /g, " ");

    // 3. 合并至一个字符串，然后导出
    let head = "表格图形链接如下（复制到浏览器中打开）:";
    let content = head + "\n\n" + pure_url + "\n\n" + text;
    createAndDownloadFile("图片地址和表格内容.txt", content);
}

/**
 * 对于文字和图形混合型的data只能存储其中的纯文字\
 * 适用性：百度文库
 */
function saveDocAndPicData() {
    // 获取文本
    let text_elements = document.getElementsByClassName("reader-word-layer");
    let texts = [];
    for (let elem of text_elements) {
        texts.push(elem.textContent);
    }
    let origin_content = texts.join("");
    // 美化后导出文本
    let content = formatText2(origin_content);
    createAndDownloadFile("纯文本文档.txt", content);
}

/**
 * 存储纯文本到本地，适用性：百度文库
 */
function saveTxtData() {
    let text_elements = document.getElementsByClassName("p-txt");
    let texts = [];
    for (let elem of text_elements) {
        texts.push(elem.textContent);
    }
    let content = texts.join("");
    createAndDownloadFile("纯文本文档.txt", content);
}

/**
 * 按文档类型，用对应方法储存数据到本地，适用性：百度文库
 * @returns 特殊情况下返回null，表示安全退出
 */
function saveData() {
    let type = detectType();
    if (type === "ppt") {
        // ppt按类似于纯图文档的方法处理
        savePPTData();
    } else if (type === "safe_quit") {
        // 安全退出
        return null;
    } else {
        let data = {};
    }
}

/**
 * 下载全部图片链接，适用性：爱问共享资料、得力文库
 * @param {string} selector 图形元素的父级元素
 */
function savePicUrls(selector) {
    let pages = document.querySelectorAll(selector);
    let pic_urls = [];

    for (let elem of pages) {
        let pic_obj = elem.children[0];
        let url = pic_obj.src;
        pic_urls.push(url);
    }
    let content = pic_urls.join("\n");
    // 启动下载
    createAndDownloadFile("urls.csv", content);
}

/**
 * 存储所有canvas图形为png到一个压缩包
 * @param {Array} node_list canvas元素列表
 * @param {String} title 文档标题
 */
function saveCanvasesToZip(node_list, title) {
    // canvas元素转为png图像
    // 所有png合并为一个zip压缩包
    let zip = new JSZip();
    let n = node_list.length;

    for (let i = 0; i < n; i++) {
        let canvas = node_list[i];
        let data_base64 = canvas.toDataURL();
        let blob = atob(data_base64.split(",")[1]);
        zip.file(`page-${i+1}.png`, blob, { binary: true });
    }

    // 导出zip
    // promise.then(onCompleted, onRejected);
    zip.generateAsync({ type: "blob" }).then(function(content) {
        // see filesaver.js
        console.log(content);
        saveAs(content, `${title}.zip`);
    });
}

/**
 * 将canvas转为jpeg，然后导出pdf
 * @param {Array} node_list canvas元素列表
 * @param {String} title 文档标题
 * @param {Number} quality 图片质量，浮点数，范围 (0,1]，默认值0.92
 */
function saveCanvasesToPDF(node_list, title, quality = 0.92) {
    let first_canvas = node_list[0];
    // 如果style的长宽不存在，则直接用canvas的元素长宽
    let width_str, height_str;
    if (!first_canvas.style.width) {
        [width_str, height_str] = [first_canvas.style.width.replace(/(px)|(rem)|(em)/, ""), first_canvas.style.height.replace(/(px)|(rem)|(em)/, "")];
    } else {
        [width_str, height_str] = [first_canvas.width, first_canvas.height];
    }
    // jsPDF的第三个参数为format，当自定义时，参数为数字数组。
    let [width, height] = [parseFloat(width_str), parseFloat(height_str)];
    // 如果文档第一页的宽比长更大，则landscape，否则portrait
    let orientation = width > height ? 'l' : 'p';
    let pdf = new jsPDF(orientation, 'px', [height, width]);

    // 保存每一页文档到每一页pdf
    node_list.forEach(function(canvas, index) {
        pdf.addImage(canvas.toDataURL("image/jpeg", quality), 'JPEG', 0, 0, width, height);
        // 如果当前不是文档最后一页，则需要添加下一个空白页
        if (index !== node_list.length - 1) {
            pdf.addPage();
        }
    });

    // 导出文件
    pdf.save(`${title}.pdf`);
}


/**
 * 创建两个初始按钮：展开文档、存储文档
 * @returns 
 */
function create2btns() {
    // 创建脚本启动按钮1、2
    let btn_1 = document.createElement("button");
    let btn_2 = document.createElement("button");

    // 设定按钮1、2样式
    btn_1.setAttribute("class", "init-btn");
    btn_1.style.height = "25px";
    btn_1.style.width = "50%";
    btn_1.style.marginLeft = "25%";
    btn_1.style.border = "none";
    btn_1.style.backgroundColor = "blue";
    btn_1.style.color = "white";
    btn_1.style.fontWeight = "bold";
    btn_1.textContent = "展开文档";
    btn_1.style.zIndex = "99999";

    btn_2.setAttribute("class", "save-doc-btn");
    btn_2.style.height = "25px";
    btn_2.style.width = "50%";
    btn_2.style.marginLeft = "25%";
    btn_2.style.backgroundColor = "green";
    btn_2.style.border = "none";
    btn_2.style.display = "none";
    btn_2.style.color = "white";
    btn_2.style.fontWeight = "bold";
    btn_2.style.zIndex = "99999";

    // 添加按钮元素到页面
    let section = document.createElement("section");
    section.setAttribute("class", "btns_section");
    section.appendChild(btn_1);
    section.appendChild(btn_2);
    document.body.appendChild(section);
    // 返回元素引用
    return [btn_1, btn_2]
}

/*
 *  主函数部分
 */

/**
 * 百度文库文档下载策略
 */
function baiduWenku() {
    // 创建脚本启动按钮1、2
    let [btn_1, btn_2] = create2btns();
    btn_2.textContent = "导出全部图片";

    // 绑定主函数
    btn_1.onclick = () => {
        enhanceBtnClickReaction();
        readAll();
    };
    btn_2.onclick = () => {
        enhanceBtnClickReaction("btn_2");
        saveData();
    };

    // 解除打印限制
    allowPrint();
}

/**
 * 豆丁文档下载策略
 */
function docin() {
    // 创建脚本启动按钮
    let [btn_1, btn_2] = create2btns();
    btn_2.textContent = "导出全部图片";

    // 隐藏底部工具栏
    document.querySelector("#j_select").click(); // 选择指针
    let tool_bar = document.querySelector(".reader_tools_bar_wrap.tools_bar_small.clear");
    tool_bar.style.display = "none";

    // 绑定主函数
    let getCanvasList = function() {
        // 获取全部canvas元素，用于传递canvas元素列表给 btn_2 和 btn_3
        let parent_node_list = document.querySelectorAll(".hkswf-content");
        let node_list = [];
        for (let node of parent_node_list) {
            node_list.push(node.firstElementChild);
        }
        return node_list;
    };

    let prepare = function() {
        // 获取canvas元素列表
        let node_list = getCanvasList();
        // 获取文档标题
        let title;
        if (document.querySelector("h1 [title=doc]")) {
            title = document.querySelector("h1 [title=doc]").nextElementSibling.textContent;
        } else if (document.querySelector(".doc_title")) {
            title = document.querySelector(".doc_title").textContent;
        } else {
            title = "文档";
        }
        // 根据页数决定图形质量
        let quality, page_num_str, page_num;
        page_num_str = document.querySelectorAll(".info_txt")[1].children[0].textContent;
        page_num = parseInt(page_num_str);

        if (page_num <= 25) {
            quality = 1.0;
        } else if (25 < page_num <= 50) {
            quality = 0.85;
        } else {
            quality = 0.7;
        }

        return [node_list, title, quality];
    }

    // 判断是否有canvas元素
    let detectCanvas = function() {
        let btn_2 = document.querySelector(".save-doc-btn");
        let haveCanvas = getCanvasList().length === 0 ? false : true;
        // 如果没有canvas元素，则认为文档页面由外链图片构成
        if (!haveCanvas) {
            // 调整按钮显示
            transformButtons(false, false, "导出全部图片链接", false);
            // 绑定主函数
            btn_2.onclick = function() {
                enhanceBtnClickReaction("btn_2");
                if (confirm("确定每页内容都加载完成了吗？")) {
                    savePicUrls("[id*=img_]");
                }
            }
        } else {
            // 调整按钮显示
            transformButtons(true, false, "", false);
            // btn_2: 导出zip
            btn_2.onclick = function() {
                enhanceBtnClickReaction("btn_2");
                if (confirm("确定每页内容都加载完成了吗？")) {
                    saveCanvasesToZip(...prepare());
                }
            };
            // btn_3: 导出pdf
            createPrintPageBtn(function() {
                if (confirm("确定每页内容都加载完成了吗？")) {
                    saveCanvasesToPDF(...prepare());
                }
            });
        }
    }
    btn_1.textContent = "判断文档类型";
    btn_1.onclick = detectCanvas;
}

/**
 * 爱问共享资料文档下载策略
 */
function ishare() {
    // 创建脚本启动按钮1、2
    let [btn_1, btn_2] = create2btns();
    btn_2.textContent = "导出全部图片链接";

    // 绑定主函数
    btn_1.onclick = readAlliShare;
    btn_2.onclick = function() { savePicUrls(".data-detail"); };

    // 移除底部下载条
    let detailfixed = document.getElementsByClassName("detail-fixed")[0];
    detailfixed.remove();
}

/**
 * 得力文库文档下载策略
 */
function deliwenku() {
    // 创建脚本启动按钮1、2
    let [btn_1, btn_2] = create2btns();
    btn_2.textContent = "导出全部图片链接";

    // 绑定主函数
    btn_1.onclick = readAllDeliwenku;
    btn_2.onclick = function() { savePicUrls('.inner_page div'); };

    // 尝试关闭页面弹窗
    try { document.querySelector("div[title=点击关闭]").click(); } catch (e) { console.log(0); }
    // 解除打印限制
    allowPrint();
}

/**
 * 道客巴巴文档下载策略
 */
function doc88() {
    // 创建脚本启动按钮1、2
    let [btn_1, btn_2] = create2btns();
    btn_2.textContent = "导出全部图片";

    // 绑定主函数
    let prepare = function() {
        // 获取canvas元素列表
        let node_list = document.querySelectorAll(".inner_page");
        // 获取文档标题
        let title;
        if (document.querySelector(".doctopic h1")) {
            title = document.querySelector(".doctopic h1").title;
        } else {
            title = "文档";
        }
        // 根据页数决定图形质量
        let quality, page_num_str, page_num;
        page_num_str = document.querySelector("#pageNumInput").parentElement.textContent.replace(" / ", "");
        page_num = parseInt(page_num_str);

        if (page_num <= 25) {
            quality = 1.0;
        } else if (25 < page_num <= 50) {
            quality = 0.85;
        } else {
            quality = 0.7;
        }

        return [node_list, title, quality];
    }

    // btn_1: 展开文档
    btn_1.onclick = readAllDoc88;
    // btn_2: 导出zip
    btn_2.onclick = function() {
        enhanceBtnClickReaction("btn_2");
        if (confirm("确定每页内容都加载完成了吗？")) {
            saveCanvasesToZip(...prepare());
        }
    };
    // btn_3: 导出pdf
    let btn_3 = createPrintPageBtn(function() {
        if (confirm("确定每页内容都加载完成了吗？")) {
            saveCanvasesToPDF(...prepare());
        }
    });
    btn_3.style.display = "none";
}

/**
 * wk文档下载策略
 */
function baiduWenkuMobile() {
    // 创建初始按钮
    clearPageWK();
    let [btn_1, btn_2] = create2btns();
    btn_2.textContent = "打印页面到PDF";
    // 绑定主函数
    btn_1.onclick = readAllWK;
    btn_2.onclick = () => {
        enhanceBtnClickReaction("btn_2");
        printPageWK();
    }
}

/**
 * 主函数：识别网站，执行对应文档下载策略
 */
function main() {
    let host = window.location.host;
    if (host === "wenku.baidu.com") {
        baiduWenku();
    } else if (host === "wk.baidu.com") {
        baiduWenkuMobile();
    } else if (host.includes("docin.com")) {
        docin();
    } else if (host === "ishare.iask.sina.com.cn") {
        ishare();
    } else if (host === "www.deliwenku.com") {
        deliwenku();
    } else if (host === "www.doc88.com") {
        doc88();
    } else {
        console.log("匹配到了无效网页");
    }
}

if (options["fast_mode"]) {
    main();
} else {
    window.onload = main;
}
